/*
* Twidget v1.1 (c) Creative Commons 2006
* http://creativecommons.org/licenses/by-sa/2.5/
* Author: Frank Manno | http://www.frankmanno.com
* Reference:    http://gettwidget.com
*               widgets@aquabox.ca
* Credits:
*       -   Keegan Jones for his ShortStat and Tada List widgets whose code inspired and helped
*           me in creating this widget.
*       -   SomeRandomDude for providing the "SansCons" icon set for public use.  Thank you!
*       -   Matthias Schmidt for helping text/debug issues preventing Twidget 2.0 from going
*           public. You rock!
*
* Changelog:
*   v2.0.5:
*           New:
*           ======
*       -   Tweet timestamps now display the date of the tweet if it is older than 1 day.
*
*           Changed:
*           =======
*       -   Changed the style of tweets in the timeline display to have a softer look with
*			dimension
*		-	Twidget no longer prevents a user from typing a tweet that exceeds 140 characters.
*			Instead, when attempting to submit a tweet, a notification is displayed advising the
*			tweet exceeds the limit, allowing the user to tailor their message as they see fit.
*
*           Fixed:
*           ======
*       -   Fixed bug that caused Twidget to stop working entirely.  This was due to the way Dashboard
*			handles <form> elements. Resolved, finally! :)
*
*   v2.0.4:
*           Fixed:
*           ======
*       -   Made sure secure transactions (anything requiring user credentials) are sent over SSL (https)
*			Thanks to Rick van der Zwet for pointing out the problem!
*		-	Fixed issue where password field would contain 4 characters and Twidget would stop working
*			altogether.
*
*   v2.0.3:   
*           Fixed:
*           ======
*       -   Removed passwords from appearing within Twidget logs (security concern)
*       -   Fixed issue where replies were no longer linking to the original message (in_reply_to_status_id)
*
*   v2.0.2:   
*           Fixed:
*           ======
*       -   Fixed "This method requires a GET" error message when manually refreshing Twidget. This
*           was due to the changes in the Twitter API made on April 8th, 2009:
*           http://apiwiki.twitter.com/REST+API+Changelog
*
*   v2.0.1:   
*           Fixed:
*           ======
*       -   Fixed bug where authenticating requests would fail with passwords containing 
*           special characters (ie: @, %, ?, etc.)
*       -   Fixed a case-sensitive reference to "AppleScrollbar.js" (was "AppleScrollBar.js")
*           which caused Twidget to fail on HFS+ systems.
*       -   Fixed bug where the interval for auto-updating the timeline would eventually lead
*           to a "rate limit exceeded" message from Twitter.  This was due to failing to reset
*           the interval upon re-setting the interval (after settings change, etc.)
*       -   Fixed bug where a broken JSON response from Twitter would cause Twidget to stop
*           working.  ** This is a workaround for bug: http://code.google.com/p/twitter-api/issues/detail?id=24 **
*
*   v2.0:   
*           New:
*           ====
*       -   Timeline (friends and public) can be toggled in main view.
*       -   Timeline state (open/closed) remember between restarts of machine.
*       -   Icons by: http://somerandomdude.net/srd-projects/sanscons/
*       -   Ability to mark/unmark tweets as favorite.
*       -   Ability to refresh the timeline and/or user's last tweet (regardless of
*           the tweet's source [ie: txt, web, etc.]).
*       -   Direct Message send support
*       -   Recognizes replies and auto-links them to follow conversations
*       -   Auto-notification of updates to Twidget
*
*           Fixed:
*           ======
*       -   Issue where loading animation would not disappear after an updated was posted
*           with Twidget under Leopard.
*       -   Fixed parsing of special characters in tweets that would cause the timeline to
*           not display at all.
*
*           Known Issues:
*           =============
*       -   Twitter has an open bug regarding "Couldn't find Status with ID=xxx" being
*           added to timeline (JSON/XML) throwing a Parse Error:
*           http://code.google.com/p/twitter-api/issues/detail?id=24
*
*   v1.1:
*       -   Fixed issue where updates containing ampersands (&) would be
*           cut off at the ampersand character.
*
*       -   Introduced updating status by hitting "Enter", in addition to 
*           clicking the "Update" button.
*
*       -   Changed character limit from 153 to the new 140 character limit
*           (as per Biz Stone's newsletter Feb. 6, 2007)
*   v1.0:   
*       -   Initial release.
*/

var Twidget = window.Twidget || {};
var YES = true;
var NO = false;

Twidget.version;
Twidget.maxChars = 140;
Twidget.showingPrefs = false;
Twidget.timelineInterval = false;

Twidget.favorites = []; // list of user's favorite twitters

// API URLs
Twidget.update_status_url_suffix = 'twitter.com/statuses/update';
Twidget.direct_message_url = 'twitter.com/direct_messages/new';
Twidget.public_timeline_url_suffix = 'twitter.com/statuses/public_timeline';
Twidget.user_timeline_url_suffix = 'twitter.com/statuses/friends_timeline';
Twidget.last_status_timeline_url = 'twitter.com/statuses/user_timeline';
Twidget.favorite_url = 'twitter.com/favorites';
Twidget.favorite_create_url = 'twitter.com/favorites/create/';
Twidget.favorite_destroy_url = 'twitter.com/favorites/destroy/';

// UI Elements
Twidget.gInfoButton = null;
Twidget.gScrollArea = null;
Twidget.gScrollBar = null;
Twidget.statusArea = null;
Twidget.loader = null;

// Are we running in debug mode?
Twidget.debug = true;

Twidget.isSetup = false;

// i18n
Twidget.lang = {};
Twidget.lang.en = {};
Twidget.lang.en.reply = "@Reply";
Twidget.lang.en.favorite = "Mark as Favorite";
Twidget.lang.en.dm = "Send a Direct Message";
Twidget.lang.en.months = [ 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec' ];

Twidget.setup = function(){
    DEBUG( "----- SETUP() -----" );
    
    var uid     = $( "uid" ),
        pwd     = $( "pwd" ),
        status  = $( "new_status_msg" ),
        bundleVersion;

    // set references to elements
    Twidget.statusArea = $( "tweetListCont" );
    Twidget.loader = $( "loader" );
    Twidget.gScrollArea = new AppleScrollArea( $( "tweetListing" ) );
    Twidget.gScrollBar = new AppleVerticalScrollbar( $( "tweetScroll" ) );
    Twidget.gScrollArea.addScrollbar(Twidget.gScrollBar);
    Twidget.gInfoButton = new AppleInfoButton(
        $( "infoButton" ),
        $( "front" ),
        "black",
        "black",
        Twidget.showPreferences
    );
        
    // attempt to retrieve saved prefs
    if (window.widget){
        $USER.init();
        
        DEBUG("--( User Preferences )--");
        DEBUG( "is authenticated? " + $USER.is_authenticated );
        DEBUG("username: " + $USER.username);
        DEBUG("password: " + Base64.encode( $USER.password ));
        DEBUG("last status: " + $USER.lastStatus);
        DEBUG("timeline type: " + $USER.preferences.timeline_type);
        DEBUG("timeline ref: " + $USER.preferences.timeline_refresh);

        bundleVersion = Twidget.getCurrentVersion();
        DEBUG("bundle version: " + bundleVersion);
        Twidget.checkForNewVersion();

        Twidget.version = bundleVersion;
        // set version label
        $( "versionTxt" ).innerHTML = "v" + Twidget.version;
        
        // Prefill form with user's existing preferences
        if ( ! $USER.username ) {
            $USER.username = "";
        } else {
            uid.value = $USER.username;
        }
        
        if ( ! $USER.password ) {
            $USER.password = "";
        } else {
            pwd.value = $USER.password;
        }

        if ( ! $USER.lastStatus ) {
            Twidget.setStatus( "" );
        } else {
            Twidget.setStatus( $USER.lastStatus );
        }
        
        if ( $USER.preferences.timeline_type ){
            var timelineType = $( "timelineTypeOptions" );
            for ( var i=0, j=timelineType.length; i < j; i++ ){
                if ( timelineType[i].value == $USER.preferences.timeline_type ){
                    timelineType[i].selected = true;
                }
            }
        }
        
        if ( $USER.preferences.timeline_refresh ){
            var timelineRefresh = $( "timelineRefreshOptions" );
            for ( var i=0, j=timelineRefresh.length; i < j; i++ ){
                if ( timelineRefresh[i].value == ( $USER.preferences.timeline_refresh / 1000 / 60 ) ){
                    timelineRefresh[i].selected = true;
                }
            }
        }

        DEBUG("--( startup end )--");
    }

    // textarea to enter twitter status
    status.addEventListener( "keyup", $UTIL.charCounter, false );
    
    // if settings have not yet been set/saved, rollover to settings pane
    if ( ! $USER.username || ! $USER.password || ! $USER.is_authenticated ){
        setTimeout ('Twidget.showPreferences();', 1000); // don't interfere with ripples
        return;
    }

    $REQUEST.loadLastStatus($USER.username);
    $REQUEST.loadFavorites($USER.username);
    
    // check whether preference is set to keep timeline display open
    DEBUG("Timeline Open: " + $USER.preferences.timeline_open );
    if ( $USER.preferences.timeline_open == true ){
        $( "arrowButton" ).addClassName( "open" );
        $( "arrowButton" ).removeClassName( "closed" );
        $( "frontBottom" ).addClassName( "hidden" );
        $( "tweetListCont" ).removeClassName( "hidden" );
    }
    
    Twidget.isSetup = true;
};

Twidget.showPreferences = function(){
    var front   = $( "front" ), 
        back    = $( "back" ),
        uid;
    
    if (window.widget){
        widget.prepareForTransition("ToBack");  // freeze widget for change
    }
    
    front.style.display = "none";   // hide front
    back.style.display = "block";   // show back
    
    Twidget.showingPrefs = true;
    
    if (window.widget){
        setTimeout ('widget.performTransition();', 0);  // and flip the widget over
    }

    if ($USER.username == "" || ! $USER.username){
        $("uid").focus();
    } else {
        $("uid").value = $USER.username.replace(/\%40/, "@");
    }
    
    if ( ($USER.password != "" || ! $USER.password) && $USER.password != null){
        $("pwd").value = decodeURIComponent($USER.password);
	}
    
    DEBUG("============= Timeline Type: " + $USER.preferences.timeline_type );
    if ($USER.preferences.timeline_type == null){
        var timelineRef = $( "timelineRefreshOptions" );
        timelineRef.disabled = true;
    }
    
    var timelineTypeOptions = $( "timelineTypeOptions" );
    timelineTypeOptions.onchange = function(e){
        Twidget.toggleTimelineSettings(timelineTypeOptions);
    };
};

Twidget.hidePreferences = function(){
    if ( $( "uid" ).value == "" || $( "pwd" ).value == "" ){
        $( "infoBack" ).innerHTML = "Please enter your credentials.";
        $( "infoBack" ).className = "error";
        $( "uid" ).select();
        return;
    } else {
        $( "infoBack" ).innerHTML = "";
        $( "infoBack" ).className = "";
        Twidget.setPreference( "isAuthenticated", true );
        Twidget.is_authenticated = true;
    }
    
    var front   = $( "front" ),
        back    = $( "back" );
    
    if (window.widget){
        widget.prepareForTransition("ToFront"); // freeze widget for change
    }
    
    back.style.display = "none";
    front.style.display = "block";
    
    Twidget.showingPrefs = false;
    if (window.widget){

        // save preferences
        var uid = $( "uid" ).value,
            pwd = $( "pwd" ).value,
            tt  = $( "timelineTypeOptions" ).value,
            tr;
        
        if ( $( "timelineRefreshOptions" ).disabled ){
            tr = 0;
        } else {
            tr = ( $( "timelineRefreshOptions" ).value * 1000 ) * 60;
        }

        if (uid != ""){ 
            Twidget.setPreference("uid", uid);
        }

        if (pwd != ""){
            Twidget.setPreference("pwd", Base64.encode( pwd ));
        }
        
        Twidget.setPreference( "timelineType", tt );
        Twidget.setPreference( "timelineRefresh", tr );
        
        $USER.preferences.timeline_type = tt;
        $USER.preferences.timeline_refresh = tr;
        $USER.username = uid.replace(/@/, "%40");
        $USER.password = encodeURIComponent(pwd);

        DEBUG("setting preference (user): " + uid );
        DEBUG("setting preference (pwd): " + pwd );
        DEBUG("setting preference (type): " + tt );
        DEBUG("setting preference (refresh): " + tr );
    } else {
        $USER.preferences.timeline_type = $( "timelineTypeOptions" ).value;
        $USER.preferences.timeline_refresh = 30000;
        $USER.username = $( "uid" ).value.replace(/@/, "%40");
        $USER.password = encodeURIComponent( $( "pwd" ).value );
    }
    
    if (window.widget){
        setTimeout('widget.performTransition();', 0);  // and flip the widget over
    }

    if ( $USER.preferences.timeline_type != "none" ){
        window.clearInterval( Twidget.timelineInterval );
        $REQUEST.loadFavorites($USER.username);
    }
};

Twidget.setPreference = function(key, value){
    return widget.setPreferenceForKey(value, $UTIL.createKey(key));
};

Twidget.getPreference = function(key){
    return widget.preferenceForKey($UTIL.createKey(key)) || false;
};

Twidget.removePreference = function( key ){
	widget.setPreferenceForKey( null, $UTIL.createKey(key) );
};

Twidget.setStatus = function(msg){
    var last = $( "lastStatus" );
    msg = msg || "";
    last.innerHTML = $UTIL.truncate(msg);
    last.title = msg || "none";
};

Twidget.replyToUser = function( event, username, tweet_id ){
    DEBUG( "replying to user: " + username );
    $( 'new_status_msg' ).value = "@" + username + ' ';
    $( 'new_status_msg' ).setSelectionRange( $( 'new_status_msg' ).value.length, $( 'new_status_msg' ).value.length );
    
    $UTIL.charCounter( { target: $( 'new_status_msg' ) } );
    Event.stop( event );
    
    if ( tweet_id ){
        Twidget.in_reply_to_status_id = tweet_id;
    }

    $( 'new_status_msg' ).blur();
    $( 'new_status_msg' ).focus();
};

Twidget.directMessage = function( event, username ){
    DEBUG( "direct messaging user: " + username );
    $( 'new_status_msg' ).value = "d " + username + " " + $( 'new_status_msg' ).value;
    $( 'new_status_msg' ).setSelectionRange( $( 'new_status_msg' ).value.length, $( 'new_status_msg' ).value.length );
    
    $UTIL.charCounter( { target: $( 'new_status_msg' ) } );
    Event.stop( event );

    $( 'new_status_msg' ).blur();
    $( 'new_status_msg' ).focus();
};

Twidget.viewProfile = function( screen_name ){
    $UTIL.openURL( "http://twitter.com/" + screen_name );
};

Twidget.viewTweet = function( screen_name, status ){
    $UTIL.openURL( "http://twitter.com/" + screen_name + "/statuses/" + status );
};

Twidget.toggleTimelineSettings = function(timelineType){
    DEBUG("Type: " + timelineType.value );
    var timelineRefresh = $( "timelineRefreshOptions" );
    switch ( timelineType.value ){
        case "none" :
            timelineRefresh.disabled = true;
            break;
        default : 
            timelineRefresh.disabled = false;
    }
};

Twidget.processTweets = function(tweetArr, appendOnly){
    DEBUG( "--> timeline updated " );

    var tweetListing = $("tweetListing");
    
    if ( ! appendOnly ){
        tweetListing.innerHTML = "";
    }

    for ( var i=0, j=tweetArr.length; i < j; i++ ){
		var str              = '',
			now              = new Date(),
			tweet            = tweetArr[i],
			tweetDate        = new Date( tweet.created_at ),
			timeSince        = now - tweetDate,
			timeSinceInHours = Math.floor( timeSince / 1000 / 60 / 60 ),
			ts;

		// tweet is more than a day old, so we'll append a date (mon/dd)
		if ( timeSinceInHours >= 24 ){
	        ts = ( tweetDate.getHours() < 10 ) ? '0' + tweetDate.getHours() : tweetDate.getHours();
	        ts += ':';
	        ts += ( tweetDate.getMinutes() < 10 ) ? '0' + tweetDate.getMinutes() : tweetDate.getMinutes();
			ts += ' on ' + Twidget.lang.en.months[ tweetDate.getMonth() ] + '. ' + tweetDate.getDate();
		// tweet occurred today
		} else {
	        ts = ( tweetDate.getHours() < 10 ) ? '0' + tweetDate.getHours() : tweetDate.getHours();
	        ts += ':';
	        ts += ( tweetDate.getMinutes() < 10 ) ? '0' + tweetDate.getMinutes() : tweetDate.getMinutes();
		}

        var reply = false;
        if ( tweet.in_reply_to_status_id ){
            // link up replies
            var regex = /^\@(\w+)\s/;
            reply = regex.exec( tweet.text );
            if ( reply ){
                var name = reply[0].split(" ")[0].substr(1);
                reply = '<a href="#" class="reply-to" onclick="Twidget.viewTweet(\''+name+'\','+tweet.in_reply_to_status_id+');return false;"">'+name+'</a>';
            }
        }

        str += '<div id="status_'+tweet.id+'" class="itemCont clearfix">';
        str += '<a href="#" onclick="Twidget.viewProfile(\''+tweet.user.screen_name+'\');return false;">';
        str += '<img src="'+tweet.user.profile_image_url+'" class="itemImage" alt="' + tweet.user.screen_name + '\'s avatar" title="' + tweet.user.screen_name + '\'s avatar" />';
        str += '</a>';
        str += '<span class="itemActions">';
        str += '<a href="#" class="statusReply" tweet_id="'+tweet.id+'" screen_name="'+tweet.user.screen_name+'" title="'+$LANG.en.reply+'"><img src="images/reply.gif" alt="'+$LANG.en.reply+'" height="16" width="16" class="action action-reply" /></a>';
        str += '<a href="#" class="statusDirect" tweet_id="'+tweet.id+'" screen_name="'+tweet.user.screen_name+'" title="'+$LANG.en.dm+'"><img src="images/mail.gif" alt="'+$LANG.en.dm+'" height="16" width="16" class="action action-directmsg" /></a>';
        str += '<a href="#" class="statusFav" tweet_id="'+tweet.id+'" screen_name="'+tweet.user.screen_name+'" title="'+$LANG.en.favorite+'"><img src="images/star.gif" alt="'+$LANG.en.favorite+'" height="16" width="16" class="action action-fav" /></a>';
        str += '</span>';
        str += '<h2 class="itemName">';
        // str += '<a href="#" onclick="Twidget.viewProfile(\''+tweet.user.screen_name+'\');return false;">'+tweet.user.screen_name+'</a>'+' <span class="timestamp">at '+ts+' via <a href="'+$UTIL.openURL(tweet.source)+'">'+ tweet.source.replace( /(href=\")+(https?\:\/\/[^\s<>$]+)/ig, "$1#\"" );+'</a></span>';
        var tweetSource = tweet.source.replace(/(href=\")+(https?\:\/\/[^\s<>\"$]+)/ig, 'href="javascript:$UTIL.openURL(\'$2\');return false;');
        
        var txt = unescape( tweet.text );

        str += '<a href="#" onclick="Twidget.viewProfile(\''+tweet.user.screen_name+'\');return false;" title="View ' +tweet.user.screen_name+ '\'s Twitter profile">'+tweet.user.screen_name+'</a>'+' <span class="timestamp">at <a href="#" onclick="Twidget.viewTweet(\''+tweet.user.screen_name+'\','+tweet.id+');return false;">'+ts+'</a> via '+tweetSource+( reply ? ' to ' + reply : '') +'</span>';
        str += '</h2>';
        str += '<p class="itemStatus">';

        // link up urls
        txt = txt.replace(/(https?\:\/\/[^\s<>$]+)/ig, '<a href="javascript:$UTIL.openURL(\'$1\')">$1</a>');
        // link up users in tweets
        txt = txt.replace(/(^| |\()\@(\w+)/ig,'$1@<a href="#" class="profile" screen_name="$2">$2</a>');

        str += txt;
        str += '</p>';
        str += '</div>';
        
        if ( appendOnly ){
            DEBUG("appending tweet");
            tweetListing.innerHTML = str + tweetListing.innerHTML;
        } else {
            tweetListing.innerHTML += str;
        }
        
        // check if status is marked as favorite
        if ( Twidget.favorites.indexOf( tweet.id ) > -1 ){
            $( tweetListing ).select( '#status_'+tweet.id+' a.statusFav' )[0].addClassName( "faved" );
        }        
    }
    
    // display arrow indicator after timeline has been loaded
    var arrow = $( "arrowButton" );
    if ( arrow.hasClassName( "hidden" ) ){
        arrow.removeClassName( "hidden" );
    }

    Twidget.gScrollArea.refresh();
};

Twidget.setTimelineInterval = function(){
    if ( ( $USER.preferences.timeline_type != "none" || $USER.preferences.timeline_type != false ) && $USER.preferences.timeline_refresh != false ){
        Twidget.updateTimeline();
        DEBUG( "Setting Interval for: " + $USER.preferences.timeline_refresh );
        clearInterval( Twidget.timelineInterval );
        Twidget.timelineInterval = setInterval( Twidget.updateTimeline, $USER.preferences.timeline_refresh );
    }
};

Twidget.updateTimeline = function(){

    // dismiss error (if visible);
    Twidget.dismissErorr();
    
    DEBUG("-----( Timeline Interval: " + $USER.preferences.timeline_refresh + " )-----");
    
    switch( $USER.preferences.timeline_type ){
        case "friends":
            $REQUEST.loadFriendsTimeline();
            break;
            
        case "public":
            $REQUEST.loadPublicTimeline();
            break;
    }
};

Twidget.showTimeline = function(){
    var animator = new AppleAnimator(500, 10);
    var animation = new AppleAnimation(0.0, 1.0, Twidget.timelineHandler);
    animator.addAnimation(animation);
    animator.oncomplete = function() {Twidget.timelineCompleteHandler();};

    if (Twidget.statusArea.className == "hidden"){
        $( "frontBottom" ).className = "hidden";
        Twidget.statusArea.className = "";
        Twidget.statusArea.style.height = 0;
        Twidget.isMaxed = false;
    } else {
        Twidget.isMaxed = true;
    }

    animator.start();
    
    var arrow = $( "arrowButton" );
    if ( arrow.hasClassName( "closed" ) ){
        arrow.removeClassName( "closed" );
        arrow.addClassName( "open" );
        
        Twidget.setPreference( "timelineOpen", true );
        $USER.preferences.timeline_open = true;
    } else {
        arrow.removeClassName( "open" );
        arrow.addClassName( "closed" );

        Twidget.setPreference( "timelineOpen", false );
        $USER.preferences.timeline_open = false;
    }
};

Twidget.timelineHandler = function(animation, current, start, finish){
    var maxHeight = 167;

    if ( ! Twidget.isMaxed ){
        Twidget.statusArea.style.height = parseInt( (current * maxHeight) ) + "px";
    } else {
        Twidget.statusArea.style.height = parseInt( ( maxHeight - (current * maxHeight) ) ) + "px";
    }
};

Twidget.timelineCompleteHandler = function(){
    if (Twidget.statusArea.offsetHeight == 0){
        $( "frontBottom" ).className = "";
        Twidget.statusArea.className = "hidden";
    }
    Twidget.gScrollArea.refresh();
};


Twidget.displayError = function(msg){
    var errorBox = document.getElementById("error");
    var dismiss = errorBox.firstChild;
    var errorMsg = document.getElementById("errorMsg");
    
    errorMsg.innerHTML = msg;
    $( "error" ).removeClassName( "hidden" );
};

Twidget.dismissErorr = function(){
    $( "error" ).addClassName( "hidden" );
    document.getElementById("errorMsg").innerHTML = "";
};

Twidget.getCurrentVersion = function(){
    return widget.system( 'sh -c "defaults read `pwd`/Info CFBundleVersion"', null ).outputString;
};

// Check for new version of Twidget
Twidget.checkForNewVersion = function(){
    var options = {
        url: "www.gettwidget.com/widgets/twidget/version.json.php",
        success: function( res ){
            var updateInfo = res;
            DEBUG( "Current Version on Server: " + updateInfo.version );
            if ( updateInfo.version > Twidget.version ){
                DEBUG( "new version available!" );
                $( "infoBack" ).innerHTML = updateInfo.msg;
                $( "infoBack" ).addClassName( "info" );
            }
        }
    };
    
    $REQUEST.load( options );
};

// User object and preferences
Twidget.User = {
    username: null,
    password: null,
    is_authenticated: false,
    lastStatus: null,
    
    // user preferences for timeline display/type
    preferences: {
        timeline_type: null,
        timeline_refresh: 0,
        timeline_open: null
    },
    
    init: function(){
		var password = Twidget.getPreference("pwd");
		if ( password ){
			password = Base64.decode( password );
		}
	
        this.is_authenticated = Twidget.getPreference("isAuthenticated");
        this.username = Twidget.getPreference("uid") || "";
        this.username = this.username.replace(/@/, "%40");
        this.password = encodeURIComponent( password || "" );
        this.preferences.timeline_type = Twidget.getPreference("timelineType");
        this.preferences.timeline_refresh = Twidget.getPreference("timelineRefresh");
        this.preferences.timeline_open = Twidget.getPreference("timelineOpen");
        
        this.lastStatus = Twidget.getPreference("lastStatus") || "";
    }
};


// Communication Object
Twidget.Request = {
    
    FORMAT: {
        JSON: ".json",
        XML: ".xml",
        RSS: ".rss"
    },
    xhr: {},
    is_requesting: false,
    _poll: {},                      // poll id cache
    _poll_interval: 50,
    _tid: 0,                        // transaction id for XHR requests
    _timeout: {},                   // timeout id cache
    _request_timeout: 60000,
    
    
    load: function(options){
        
        var method = options.method || "POST";
        var qsArr = [];
        var qs;

        for ( param in options.params ){
            qsArr.push( param + '=' + options.params[ param ] );
        }
        
        qsArr.push( "ts=" + (new  Date).getTime() );

        qs = qsArr.join( "&" );

        if ( qsArr.length > 1 && method != "POST" ){
            options.url += "?" + qs;
        }

        // increment counter
        var tid = ++this._tid;
        var xhr = new XMLHttpRequest();
        xhr._url = options.url;
        
        this.xhr[ tid ] = xhr;

        if ( options.auth ){
            DEBUG( "Authenticated request for user: " + $USER.username );
            options.url = "https://" + $USER.username + ":" + $USER.password + "@" + options.url;
        } else {
            options.url = "http://" + options.url;
        }

        xhr.open( method, options.url, true );
        xhr.setRequestHeader( "Cache-Control", "no-cache" );

        if ( options.params && method == "POST" ){
            xhr.setRequestHeader( "Method", "POST "+options.url+" HTTP/1.1");
            xhr.setRequestHeader( "Content-type", "application/x-www-form-urlencoded" );
            xhr.setRequestHeader( "Content-length", qs.length );
        }

        DEBUG( "Method: " + method );
        DEBUG("QS: " + qs);

        // assign transaction id
        xhr._tid = tid;

        // polling for readystate rather than using "onreadystatechange"
        // (a page taken from YUI... thanks for the heads-up on an 
        // alternative method!)
        this.handleReadyState.call( this, xhr, options );

        xhr.send( qs || "" );

        this.is_requesting = true;

        $UTIL.showLoader();
    },
    
    handleReadyState: function(xhr, options){

        DEBUG( "url: " + xhr._url );

        // set timeout and call failure callback
        var that = this;
        this._timeout[ xhr._tid ] = window.setTimeout( function(){
            LOG( "request timeout: " + xhr._tid );
            LOG( "timeout url: " + xhr._url );
            xhr.abort();

            window.clearInterval( that._poll[ xhr._tid ] );

            if ( options.failure ){
                options.failure( { error: "An error occurred.  Are you sure you're connected?" } );
            }

            delete that._poll[ xhr._tid ];
            delete that._timeout[ xhr._tid ];
            delete that.xhr[ xhr._tid ];

            that.is_requesting = false;

            $UTIL.hideLoader();
        }, this._request_timeout );

        this._poll[ xhr._tid ] = window.setInterval(
            function(){
                if ( xhr.readyState === 4 ) {

                    // delete timeouts/intervals that may exist
                    window.clearInterval( that._poll[ xhr._tid ] );
                    window.clearTimeout( that._timeout[ xhr._tid ] );
                    delete that._poll[ xhr._tid ];
                    delete that._timeout[ xhr._tid ];
                    delete that.xhr[ xhr._tid ];

                    if( xhr.status >= 200 && xhr.status < 300 ) {

                        // Workaround for invalid JSON response from Twitter
                        var regexStatus = /Couldn't find Status with ID=([0-9]*)/,
                            regexUser   = /Couldn't find User with ID=([0-9]*)/,
                            response    = xhr.responseText;

                        response = response.replace(regexStatus, '', 'g').replace(regexUser, '', 'g');
                        response = response.evalJSON( true );

                        if ( response.error ){

                            LOG("Response:");
                            LOG(xhr.responseText);

                            options.failure(response);
                        } else {
                            options.success(response);
                        }
                    } else {

                        LOG("Error loading url: " + xhr._url + " with status: "+ xhr.status);
                        LOG("Ready State: " + xhr.readyState );
                        LOG("Response:");
                        LOG(xhr.responseText);

                        var errorObj;
                        if ( xhr.responseText ){
                            errorObj = xhr.responseText.evalJSON( true );
                        } else {
                            var msg;
                            switch( xhr.status ){
                                case 0:
                                    msg = "An error occurred.  Are you sure you're connected?";
                                    break;
                                default:
                                    msg = "An error occurred.  Try again.";
                                    break;
                            }
                            
                            errorObj = { error: msg };
                        }

                        if ( options.failure ){
                            options.failure(errorObj);
                        }
                    }

                    $UTIL.hideLoader();
                    that.is_requesting = false;

                } else {
                    //DEBUG ("Not in readystate 4: "+ xhr.readyState);
                }
            },
            this._poll_interval
        );
    },
    
    loadPublicTimeline: function(){
        if ( $USER.username != "" && $USER.password != "" ){
            var options = {
                url : Twidget.public_timeline_url_suffix + this.FORMAT.JSON,
                method: "GET",
                success: this.loadPublicTimelineSuccess,
                failure: this.loadPublicTimelineFailure
            };

            this.load(options);
        }
    },
    
    loadPublicTimelineSuccess: function(r){
        DEBUG(" ---( loadPublicTimelineSuccess )---");
        Twidget.processTweets( r );
    },

    loadPublicTimelineFailure: function(r){
        LOG(" ---( loadPublicTimelineFailure )---");
        LOG(r.error);
        Twidget.displayError(r.error);
    },
    
    loadFriendsTimeline: function(){
        if ( $USER.username != "" && $USER.password != "" ){
            var options = {
                auth: YES,
                method: "GET",
                url : Twidget.user_timeline_url_suffix + this.FORMAT.JSON,
                success: this.loadFriendsTimelineSuccess,
                failure: this.loadFriendsTimelineFailure
            };
            
            this.load(options);
        }
    },
    
    loadFriendsTimelineSuccess: function(r){
        DEBUG(" ---( loadFriendsTimelineSuccess )---");
        DEBUG("Tweet Length: " + r.length );
        Twidget.processTweets( r );
    },

    loadFriendsTimelineFailure: function(r){
        LOG(" ---( loadFriendsTimelineFailure )---");
        LOG(r.error);
        Twidget.displayError(r.error);
    },
    
    loadFavorites: function(user){
        var options = {
            url: Twidget.favorite_url + this.FORMAT.JSON,
            auth: YES,
            method: "GET",
            success: this.loadFavoritesSuccess,
            failure: this.loadFavoritesFailure
        };
        
        this.load(options);
    },
    
    loadFavoritesSuccess: function(r){
        DEBUG(" ---( loadFavoritesSuccess )---");
        
        var favorites = r;

        for ( var i = 0, j = favorites.length; i < j; i++ ){
            Twidget.favorites.push( favorites[i].id );
        }

        // load timeline
        Twidget.setTimelineInterval();
    },

    loadFavoritesFailure: function(r){
        LOG(" ---( loadFavoritesFailure )---");
        LOG(r.error);

        // load timeline
        Twidget.setTimelineInterval();
    },
    
    loadLastStatus: function(user){
        var options = {
            url: Twidget.last_status_timeline_url + this.FORMAT.JSON + "?count=1",
            auth: YES,
            method: "GET",
            success: this.loadLastStatusSuccess,
            failure: this.loadLastStatusFailure
        };

        this.load(options);
    },
    
    loadLastStatusSuccess: function(r){
        DEBUG(" ---( loadLastStatusSuccess )---");
        
        if ( !r[0] ){ return; }

        var update = r[0].text;
        Twidget.setStatus( update );

        // save preference
        Twidget.setPreference( "lastStatus", update );
    },

    loadLastStatusFailure: function(r){
        LOG(" ---( loadLastStatusFailure )---");
        LOG(r.error);
        Twidget.displayError(r.error);
    },
    
    updateStatus: function(status){
        
        if ( !status || !status.length || status.trim().length == 0 ){
            return;
        }

		// check for character limits
		if ( status.length > Twidget.maxChars ){
			Twidget.displayError( "Your tweet is more than " + Twidget.maxChars + " characters long. Try shortening it." );
			return;
		}
        
        // check to see if we're sending a direct msg
        Twidget.isDirectMsg = /^(d|D) /.test(status);

        if ( Twidget.isDirectMsg ){
            var dmRegEx = /^(d|D)\s[a-zA-Z0-9]+\s/;
            var u = dmRegEx.exec( status );
            u = u[0].split(" ")[1]; 
            DEBUG("User Matched: " + u);
            var msg = status.replace(dmRegEx, "");

            var options = {
                url: Twidget.direct_message_url + this.FORMAT.JSON,
                auth: YES,
                params: {
                    user: u,
                    text: encodeURIComponent(msg),
                    source: "twidget"
                },
                success: this.directMessageSuccess,
                failure: this.directMessageFailure
            };


            DEBUG("Direct Message: " + msg + " to user: " + u);
            DEBUG("Updating status with message: " + msg );
            DEBUG( "Encoded: " + encodeURIComponent(msg) );
            
        } else {
            var options = {
                url: Twidget.update_status_url_suffix + this.FORMAT.JSON,
                auth: YES,
                params: {
                    status: encodeURIComponent(status),
                    source: "twidget"
                },
                success: this.updateStatusSuccess,
                failure: this.updateStatusFailure
            };
            
            if ( Twidget.in_reply_to_status_id ){
                LOG( "replying to status: " + Twidget.in_reply_to_status_id );
                options.params.in_reply_to_status_id = Twidget.in_reply_to_status_id;
            }

            DEBUG("Updating status with message: " + status );
            DEBUG( "Encoded: " + encodeURIComponent(status) );
        }

        this.load(options);
        Twidget.in_reply_to_status_id = null;
    },
    
    updateStatusSuccess: function(r){
        DEBUG(" ---( updateStatusSuccess )---");
        var update = r.text;
        Twidget.setStatus( update );

        // save preference
        Twidget.setPreference( "lastStatus", update );

        Twidget.processTweets([r], true);

        // clear loader, input, and char counter
        $( "new_status_msg" ).value = "";
        $( "charsLeft" ).innerHTML = Twidget.maxChars;
    },

    updateStatusFailure: function(r){
        LOG(" ---( updateStatusFailure )---");
        LOG(r.error);
        Twidget.displayError(r.error);
    },
    
    directMessageSuccess: function(r){
        DEBUG(" ---( directMessageSuccess )---");
        var update = r.text;
        r.source = 'twidget';
		r.user = r.recipient;
		r.truncated = 0;
		r.isreply = 0;
		r.isfav = 0;
		r.type = 'direct';
		
        // Twidget.setStatus( update );
        
        Twidget.processTweets([r], true);
        
        // clear loader, input, and char counter
        $( "new_status_msg" ).value = "";
        $( "charsLeft" ).innerHTML = Twidget.maxChars;
    },
    
    directMessageFailure: function(r){
        LOG(" ---( directMessageFailure )---");
        LOG(r.error);
        Twidget.displayError(r.error);
    },
    
    markAsFavorite: function( event, statusId ){
        var favStar = $( "status_" + statusId ).select( 'a.statusFav' )[0];

        var options = {
            auth: YES
        };

        if ( $( favStar ).hasClassName( "faved" ) ){
            // var twitterUrl = "".concat( "https://", Twidget.username, ":", Twidget.password, "@", Twidget.favorite_destroy_url, statusId, Twidget.request_format );
            options.url = Twidget.favorite_destroy_url + statusId + this.FORMAT.JSON;
            options.success = this.unmarkAsFavoriteSuccess;
            options.failure = this.markAsFavoriteFailure;
        } else {
            // var twitterUrl = "".concat( "https://", Twidget.username, ":", Twidget.password, "@", Twidget.favorite_create_url, statusId, Twidget.request_format );
            options.url = Twidget.favorite_create_url + statusId + this.FORMAT.JSON;
            options.success = this.markAsFavoriteSuccess;
            options.failure = this.markAsFavoriteFailure;
        }

        this.load(options);
    },
    
    markAsFavoriteSuccess: function(r){
        DEBUG(" ---( markAsFavoriteSuccess )---");
        var favStar = $( "status_" + r.id ).select( 'a.statusFav' )[0].addClassName( "faved" );
    },
    
    unmarkAsFavoriteSuccess: function(r){
        DEBUG(" ---( unmarkAsFavoriteSuccess )---");
        var favStar = $( "status_" + r.id ).select( 'a.statusFav' )[0].removeClassName( "faved" );
    },

    markAsFavoriteFailure: function(r){
        LOG(" ---( markAsFavoriteFailure )---");
        LOG(r.error);
        Twidget.displayError(r.error);
    }
};

// Truncate string and append ellipsis (...)
Twidget.util = {

    // Create unique identifier for Twidget
    createKey: function(key){
        return widget.identifier + '-' + key;
    },

    truncate: function(val){
        if ( ! val || val == "" ){
            return "";
        }

        var MAXLEN = 54;
        if (val.length > MAXLEN){
            return val.substring(0, MAXLEN-3).concat("\u2026");
        }

        return val;
    },

    // Count characters in status textarea
    charCounter: function(e){
        var input = e.target;
        $( "charsLeft" ).innerHTML = Twidget.maxChars - input.value.length;
    },
    
    // If Enter key is pressed while on the back side, flip back to front.
    handleKey: function() {
        var keyCode = event.keyCode ? event.keyCode : event.which ? event.which : event.charCode;
        if (keyCode == 13 && Twidget.showingPrefs) {
            Twidget.hidePreferences();
        } else if (keyCode == 13 && !Twidget.showingPrefs){
            $REQUEST.updateStatus( $( 'new_status_msg' ).value );
            return false;
        }
    },
    
    allowInput: function( input ){
        // check to see whether the message is a reply (starts with @) or
        // a direct message (starts with d).  The character count should
        // not take these directives into account
        var isDirectMsg = /^(d|D)\s[a-zA-Z0-9]+\s/.test(input.value);
        var isReply = /^@[a-zA-Z0-9]+ /.test(input.value);
		var specialKeyPressed = false;

        DEBUG("isReply: " + isReply);
        DEBUG("isDirectMsg: " + isDirectMsg);
		
		if ( input.value.length < Twidget.maxChars && ! specialKeyPressed ){
			allow = false;
		}
    },

    // Open Link in Default System Browser
    openURL: function(url) {
        DEBUG( "Opening URL: " + url );
        if (window.widget) {
            widget.openURL(url);
        } else {
            window.location.href = url;
        }
    },

    showLoader: function(){
        $(Twidget.loader).removeClassName("hidden");
    },

    hideLoader: function(){
        $(Twidget.loader).addClassName("hidden");
    },

 	destroy: function(){
		// Stop any timers to prevent CPU usage
	    // Remove any preferences as needed
		if (window.widget){
			
			// only remove the widget specific keys.
			Twidget.removePreference( "isAuthenticated" );
			Twidget.removePreference( "uid" );
			Twidget.removePreference( "pwd" );
			Twidget.removePreference( "timelineType" );
			Twidget.removePreference( "timelineRefresh" );
			Twidget.removePreference( "timelineOpen" );
			Twidget.removePreference( "lastStatus" );
			
			// clear timers
			clearInterval( Twidget.timelineInterval );
		}
	}
};

function clickDelegate( e ){

    if ( !e && !e.target ) return;
    var t = e.target;

    if ( t.id == "logout" ){
        Twidget.endSession();
    }

    if ( t.id == "arrowButton" ){
        Twidget.showTimeline();
        Event.stop( e );
        return false;
    }

    if ( t.hasClassName( "action-reply" ) ){
        t = t.parentNode;
        var screen_name = t.getAttribute( "screen_name" );
        var tweet_id = t.getAttribute( "tweet_id" );
        Twidget.replyToUser( e, screen_name, tweet_id );
        return false;
    }
    
    if ( t.hasClassName( "action-fav" ) ){
        t = t.parentNode;
        var tweet_id = t.getAttribute( "tweet_id" );
        $REQUEST.markAsFavorite( e, tweet_id );
        return false;
    }
    
    if ( t.hasClassName( "action-directmsg" ) ){
        t = t.parentNode;
        var screen_name = t.getAttribute( "screen_name" );
        Twidget.directMessage( e, screen_name );
        return false;
    }
    
    if ( t.hasClassName( "profile" ) ){
        var screen_name = t.getAttribute( "screen_name" );
        Twidget.viewProfile( screen_name );
        return false;
    }
    
    if ( t.hasClassName( "refresh" ) ){
        if ( $USER.preferences.timeline_type != "none" && $USER.preferences.timeline_refresh > 0 ){
            $REQUEST.loadFavorites( $USER.username );
            Twidget.updateTimeline();
        }
        $REQUEST.loadLastStatus( $USER.username );
        return false;
    }

    if ( t.hasClassName( "dismiss" ) && t.parentNode.id == "error" ){
        Twidget.dismissErorr();
        return false;
    }

}

var $LANG = Twidget.lang;
var $USER = Twidget.User;
var $REQUEST = Twidget.Request;
var $UTIL = Twidget.util;

String.prototype.trim = function() { 
    return this.replace(/^\s*/, "").replace(/\s*$/, "");
};

document.onkeydown = $UTIL.handleKey;

Event.observe( window, 'load', function() {
    Event.observe( document, 'click', clickDelegate );
} );

if ( window.widget ){
	widget.onremove = function(){
		alert( "removing Twidget" );
		$UTIL.destroy();
	};
}